/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc;

import java.util.*;
import irc.*;
import irc.gui.pixx.*;
import java.awt.*;

class ServerItem
{
  public String host;
  public int port;
  public String pass;
}

/**
 * Toolkit for Configuration creation.
 */
public class ConfigurationLoader
{
  private ParameterProvider _provider;
  private URLHandler _handler;
  private ImageLoader _loader;
  private SoundHandler _sound;
  private FileHandler _file;

  /**
   * Create a new IRCConfigurationLoader.
   * @param provider parameter provider to load the configuration from.
   * @param handler URL handler.
   * @param loader Image loader.
   * @param sound Sound handler.
   * @param file File handler.
   */
  public ConfigurationLoader(ParameterProvider provider,URLHandler handler,ImageLoader loader,SoundHandler sound,FileHandler file)
  {
    _provider=provider;
    _handler=handler;
    _loader=loader;
    _sound=sound;
    _file=file;
  }

  /**
   * Create a new IRCConfiguration object, using the given ParameterProvider.
   * @return a new IRCConfiguration instance.
   * @throws Exception if an error occurs.
   */
  public IRCConfiguration loadIRCConfiguration() throws Exception
  {
    return getIRCConfiguration();
  }

  /**
   * Create a new StartupConfiguration object, using the given ParameterProvider.
   * @return a new StartupConfiguration instance.
   * @throws Exception if a mandatory parameter is not supplied or if an error occurs.
   */
  public StartupConfiguration loadStartupConfiguration() throws Exception
  {
    return getStartupConfiguration();
  }

  private String getParameter(String key)
  {
    return _provider.getParameter(key);
  }

  private boolean getBoolean(String key,boolean def)
  {
    String v=getParameter(key);
    if(v==null) return def;
    v=v.toLowerCase().trim();
    if(v.equals("true") || v.equals("on") || v.equals("1")) return true;
    return false;
  }

  private String getString(String key,String def)
  {
    String v=getParameter(key);
    if(v==null) return def;
    return v;
  }

  private int getInt(String key,int def)
  {
    String v=getParameter(key);
    if(v==null) return def;
    try
    {
      return Integer.parseInt(v);
    }
    catch(Exception e)
    {
      return def;
    }

  }

  private TextProvider getTextProvider()
  {
    String lang=getString("language","english");
    String encoding=getString("languageencoding","");
    String extension=getString("lngextension","lng");
    String backlang=getString("backuplanguage","english");
    String backencoding=getString("backuplanguageencoding","");
    return new FileTextProvider(lang+"."+extension,encoding,backlang+"."+extension,backencoding,_file);
  }

  private String[] getArray(String name)
  {
    Vector v=new Vector();
    String cmd;
    int i=1;
    do
    {
      cmd=getParameter(name+i);
      if(cmd!=null) v.insertElementAt(cmd,v.size());
      i++;
    }
    while(cmd!=null);
    String[] ans=new String[v.size()];
    for(i=0;i<v.size();i++) ans[i]=(String)v.elementAt(i);
    return ans;
  }

  private void readSmileys(IRCConfiguration config)
  {
    String[] arr=getArray("smiley");
    for(int i=0;i<arr.length;i++)
    {
      String cmd=arr[i];
      int pos=cmd.indexOf(" ");
      if(pos!=-1)
      {
        String match=cmd.substring(0,pos).trim();
        String file=cmd.substring(pos+1).trim();
        config.addSmiley(match,file);
      }
    }
  }

  private void readChannelFont(IRCConfiguration config) throws Exception
  {
    String param=getString("channelfont","12 Monospaced");
    int pos=param.indexOf(' ');
    if(pos<0) throw new Exception("channelfont property syntax error");
    String size=param.substring(0,pos).trim();
    String name=param.substring(pos+1).trim();
    config.set("ChannelFontSize",new Integer(size).intValue());
    config.set("ChannelFontName",name);
  }

  private void readChanlistFont(IRCConfiguration config) throws Exception
  {
    String param=getString("channelfont","12 Monospaced");
    int pos=param.indexOf(" ");
    if(pos<0) throw new Exception("channelfont property syntax error");
    String size=param.substring(0,pos).trim();
    String name=param.substring(pos+1).trim();
    config.set("ChanlistFontSize",new Integer(size).intValue());
    config.set("ChanlistFontName",name);
  }

  private IRCColorModel getIRCColorModel()
  {
    PixxColorModel model;

    String base=getParameter("basecolor");
    if(base!=null)
    {
      try
      {
        int pos=base.indexOf(",");
        int r=Integer.parseInt(base.substring(0,pos));
        base=base.substring(pos+1);
        pos=base.indexOf(",");
        int g=Integer.parseInt(base.substring(0,pos));
        base=base.substring(pos+1);
        int b=Integer.parseInt(base);
        model=new PixxColorModel(r,g,b);
      }
      catch(Exception e)
      {
        e.printStackTrace();
        model=new PixxColorModel();
      }
    }
    else
    {
      model=new PixxColorModel();
    }

    for(int i=0;i<model.getColorCount();i++)
    {
      String color=getParameter("color"+i);
      try
      {
        if(color!=null)
        {
          model.setColor(i,new Color(Integer.parseInt(color,16)));
        }
      }
      catch(Exception e)
      {
      }
    }
    return model;
  }

  private void configureTextColors(IRCConfiguration config)
  {
    //type name {index=value}*
    String[] arr=getArray("sourcecolorrule");
    for(int i=0;i<arr.length;i++)
    {
      StringTokenizer tok=new StringTokenizer(arr[i]);
      if(!tok.hasMoreElements()) continue;
      String type=(String)tok.nextElement();
      if(!tok.hasMoreElements()) continue;
      String name=(String)tok.nextElement();
      Color[] c=new Color[16];
      config.loadDefaultColors(c);
      while(tok.hasMoreElements())
      {
        String s=(String)tok.nextElement();
        int pos=s.indexOf('=');
        if(pos<0) continue;
        String before=s.substring(0,pos).trim();
        String after=s.substring(pos+1).trim();
        int index=Integer.parseInt(before);
        Color col=new Color(Integer.parseInt(after,16));
        if((index>=0) && (index<=15)) c[index]=col;
      }
      config.setSourceColor(type,name,c);
    }

    /** 1.9.0 compatibility with older versions **/
    boolean has=false;
    for(int i=0;i<16;i++) if(getParameter("chanlisttextcolor"+i)!=null) has=true;
    if(has)
    {
      Color[] c=new Color[16];
      config.loadDefaultColors(c);
      for(int i=0;i<16;i++)
      {
        String color=getParameter("chanlisttextcolor"+i);
        if(color!=null) c[i]=new Color(Integer.parseInt(color,16));
      }
      config.setSourceColor("none+ChanList","all",c);
    }
    has=false;
    for(int i=0;i<16;i++) if(getParameter("defaultsourcetextcolor"+i)!=null) has=true;
    if(has)
    {
      Color[] c=new Color[16];
      config.loadDefaultColors(c);
      for(int i=0;i<16;i++)
      {
        String color=getParameter("defaultsourcetextcolor"+i);
        if(color!=null) c[i]=new Color(Integer.parseInt(color,16));
      }
      config.setSourceColor("all-ChanList","all",c);
    }
    if(getParameter("sourcecolorconfig1")!=null)
    {
      Hashtable colors=new Hashtable();
      arr=getArray("sourcecolorconfig");
      for(int i=0;i<arr.length;i++)
      {
        String cmd=arr[i];
        try
        {
          int pos=cmd.indexOf(' ');
          String source=cmd.substring(0,pos).trim();
          cmd=cmd.substring(pos+1).trim();
          pos=cmd.indexOf(' ');
          String index=cmd.substring(0,pos).trim();
          String color=cmd.substring(pos+1).trim();
          int ind=new Integer(index).intValue();
          Color col=new Color(Integer.parseInt(color,16));
          source=source.toLowerCase();
          if(colors.get(source)==null)
          {
            Color[] cols=new Color[16];
            config.loadDefaultColors(cols);
            cols[ind]=col;
            colors.put(source,cols);
          }
          else
          {
            Color[] cols=(Color[])colors.get(source);
            cols[ind]=col;
          }
        }
        catch(Exception e)
        {
        }
      }
      Enumeration e=colors.keys();
      while(e.hasMoreElements())
      {
        String key=(String)e.nextElement();
        Color[] cols=(Color[])colors.get(key);
        config.setSourceColor("all",key,cols);
      }

    }

  }

  private String[] getHighLightWords()
  {
    String words=getParameter("highlightwords");
    if(words==null) return new String[0];
    return new StringParser().parseString(words);
  }

  private void readSound(IRCConfiguration config)
  {
    AudioConfiguration ac=config.getAudioConfiguration();
    if(getParameter("soundbeep")!=null) ac.setBeep(getParameter("soundbeep"));
    if(getParameter("soundquery")!=null) ac.setQuery(getParameter("soundquery"));
    String[] arr=getArray("soundword");
    for(int i=0;i<arr.length;i++)
    {
      String cmd=arr[i];
      cmd=cmd.trim();
      int pos=cmd.indexOf(' ');
      if(pos!=-1)
      {
        String word=cmd.substring(0,pos).trim();
        String sound=cmd.substring(pos+1).trim();
        ac.setWord(word,sound);
      }
    }
  }

  private void readDocking(IRCConfiguration config)
  {
    StringParser parser=new StringParser();
    String[] arr=getArray("dockingconfig");
    for(int i=0;i<arr.length;i++)
    {
      String cmd=arr[i];
      String back[]=parser.parseString(cmd);
      if(back.length>=3)
      {
        String type=back[0].toLowerCase();
        String name=back[1].toLowerCase();
        String action=back[2].toLowerCase();
        boolean undock=false;
        if(action.equals("undock")) undock=true;
        config.setDockingPolicy(type,name,undock);
      }
    }
  }

  private void readBackgroundConfig(IRCConfiguration config)
  {
    StringParser parser=new StringParser();
    String[] arr=getArray("backgroundimage");
    for(int i=0;i<arr.length;i++)
    {
      String cmd=arr[i];
      String back[]=parser.parseString(cmd);
      if(back.length>=4)
      {
        String type=back[0];
        String name=back[1];
        int tiling=new Integer(back[2]).intValue();
        String image=back[3];
        config.setBackgroundImage(type,name,image);
        config.setBackgroundTiling(type,name,tiling);
      }
      else if(back.length>=3)
      {
        /** 1.9.0 compatibility with older versions **/
        String name=back[0];
        String type="all";
        int tiling=new Integer(back[1]).intValue();
        String image=back[2];
        config.setBackgroundImage(type,name,image);
        config.setBackgroundTiling(type,name,tiling);
      }
    }
  }

  private void add(Vector v,Object o)
  {
    v.insertElementAt(o,v.size());
  }

  private void str2(Vector v,IRCConfiguration c,int id,String str)
  {
    add(v,new String[] {c.getText(id),str});
  }

  public void loadDefaultPopup(IRCConfiguration config)
  {
    Vector v=config.getNickMenuVector();
    String[] str;

    boolean b=config.getB("ShowStatus") && !config.getB("UseInfo");
    if(b) str2(v,config,TextProvider.GUI_WHOIS,"/whois %1");
    str2(v,config,TextProvider.GUI_QUERY,"/query %1");
    str2(v,config,TextProvider.GUI_KICK,"/kick %2 %1");
    str2(v,config,TextProvider.GUI_BAN,"/mode %2 +b %1");
    str=new String[4];
    str[0]=config.getText(TextProvider.GUI_KICKBAN);
    str[1]="/mode %2 -o %1";
    str[2]="/mode %2 +b %1";
    str[3]="/kick %2 %1";
    add(v,str);

    add(v,new String[] {"--"});
    str2(v,config,TextProvider.GUI_OP,"/mode %2 +o %1");
    str2(v,config,TextProvider.GUI_DEOP,"/mode %2 -o %1");
    str2(v,config,TextProvider.GUI_VOICE,"/mode %2 +v %1");
    str2(v,config,TextProvider.GUI_DEVOICE,"/mode %2 -v %1");
    if(b)
    {
      add(v,new String[] {"--"});
      str2(v,config,TextProvider.GUI_PING,"/ctcp ping %1");
      str2(v,config,TextProvider.GUI_VERSION,"/ctcp version %1");
      str2(v,config,TextProvider.GUI_TIME,"/ctcp time %1");
      str2(v,config,TextProvider.GUI_FINGER,"/ctcp finger %1");
    }
  }

  private void readPopup(IRCConfiguration config)
  {
    if(!getBoolean("configurepopup",false))
    {
      loadDefaultPopup(config);
      return;
    }
    Vector v=config.getNickMenuVector();
    String[] str=getArray("popupmenustring");
    for(int i=0;i<str.length;i++)
    {

      String cmd[]=getArray("popupmenucommand"+(i+1)+"_");
      String[] fin=new String[1+cmd.length];
      fin[0]=str[i];
      for(int j=0;j<cmd.length;j++) fin[j+1]=cmd[j];
      add(v,fin);
    }
  }

  private void readMouse(IRCConfiguration config,String name,int button,int click) throws Exception
  {
    String s=getString("mouse"+name.toLowerCase(),button+" "+click);
    int pos=s.indexOf(' ');
    if(pos<0) throw new Exception("mouse"+name.toLowerCase()+" parameter syntax error");
    String before=s.substring(0,pos).trim();
    String after=s.substring(pos+1).trim();
    config.setMouseConfiguration(name,new Integer(before).intValue(),new Integer(after).intValue());
  }

  private IRCConfiguration getIRCConfiguration() throws Exception
  {
    IRCConfiguration config=new IRCConfiguration(getTextProvider(),getIRCColorModel(),_handler,_loader,_sound,_file);


    Vector v=new Vector();
    String[] words=getHighLightWords();
    for(int i=0;i<words.length;i++) v.insertElementAt(words[i],v.size());
    config.setHighLightConfig(getInt("highlightcolor",5),getBoolean("highlightnick",false),v);

    config.setJoinList(getString("authorizedjoinlist",""));
    config.setLeaveList(getString("authorizedleavelist",""));
    config.setCommandList(getString("authorizedcommandlist",""));

    config.set("highlight",getBoolean("highlight",false));
    config.set("timestamp",getBoolean("timestamp",false));
    config.set("smileys",getBoolean("smileys",false));
    config.set("quitmessage",getString("quitmessage",""));
    config.set("asl",getBoolean("asl",false));
    config.set("aslmale",getString("aslmale","m"));
    config.set("aslfemale",getString("aslfemale","f"));
    config.set("floatingasl",getBoolean("floatingasl",false));
    config.set("floatingaslalpha",getInt("floatingaslalpha",170));
    config.set("showstatus",getBoolean("showstatus",true));
    config.set("showclose",getBoolean("showclose",true));
    config.set("showconnect",getBoolean("showconnect",true));
    config.set("showchanlist",getBoolean("showchanlist",true));
    config.set("showabout",getBoolean("showabout",true));
    config.set("showhelp",getBoolean("showhelp",true));
    config.set("bitmapsmileys",getBoolean("bitmapsmileys",false));
    config.set("nicklistwidth",getInt("nicklistwidth",130));
    config.set("useinfo",getBoolean("useinfo",false));
    config.set("nickfield",getBoolean("nickfield",false));
    config.set("styleselector",getBoolean("styleselector",false));
    config.set("setfontonstyle",getBoolean("setfontonstyle",false));
    config.set("backgroundimage",getBoolean("backgroundimage",false));
    config.set("coding",getInt("coding",1));
    config.set("userid",getString("userid",""));
    config.set("righttoleft",getBoolean("righttoleft",false));
    config.set("automaticqueries",getBoolean("automaticqueries",true));
    config.set("autoconnection",getBoolean("autoconnection",true));
    config.set("useidentserver",getBoolean("useidentserver",true));
    config.set("helppage",getString("helppage",null));
    config.set("multiserver",getBoolean("multiserver",false));
    config.set("noasldisplayprefix",getString("noasldisplayprefix",""));
    config.set("aslunknown",getString("aslunknown","x"));
    config.set("showchannelnickchanged",getBoolean("showchannelnickchanged",true));
    config.set("showchannelnickmodeapply",getBoolean("showchannelnickmodeapply",true));
    config.set("showchannelmodeapply",getBoolean("showchannelmodeapply",true));
    config.set("showchanneltopicchanged",getBoolean("showchanneltopicchanged",true));
    config.set("showchannelnickquit",getBoolean("showchannelnickquit",true));
    config.set("showchannelnickkick",getBoolean("showchannelnickkick",true));
    config.set("showchannelnickpart",getBoolean("showchannelnickpart",true));
    config.set("showchannelnickjoin",getBoolean("showchannelnickjoin",true));
    config.set("showdock",getBoolean("showdock",true));

    readMouse(config,"NickQuery",1,2);
    readMouse(config,"URLOpen",1,2);
    readMouse(config,"ChannelJoin",1,2);
    readMouse(config,"NickPopup",3,1);
    readMouse(config,"TaskBarPopup",3,1);

    readBackgroundConfig(config);
    readSmileys(config);
    readChannelFont(config);
    readChanlistFont(config);
    configureTextColors(config);
    readSound(config);
    readPopup(config);
    readDocking(config);

    /** 1.9.0 compatibility with older versions **/
    if(getParameter("URLClickCount")!=null)
      config.setMouseConfiguration("URLOpen",1,getInt("URLClickCount",2));
    if(getParameter("defaultbackgroundimage")!=null)
      config.setBackgroundImage("all-ChanList","all",getParameter("defaultbackgroundimage"));
    if(getParameter("chanlistbackgroundimage")!=null)
      config.setBackgroundImage("none+ChanList","all",getParameter("chanlistbackgroundimage"));
    if(getParameter("defaultbackgroundtiling")!=null)
      config.setBackgroundTiling("all-ChanList","all",getInt("defaultbackgroundtiling",0));
    if(getParameter("chanlistbackgroundtiling")!=null)
      config.setBackgroundTiling("none+ChanList","all",getInt("chanlistbackgroundtiling",0));

    return config;
  }

  public ServerItem[] readServers(String dhost,int dport,String dpass)
  {
    Vector res=new Vector();
    ServerItem item=new ServerItem();
    item.host=dhost;
    item.port=new Integer(dport).intValue();
    item.pass=dpass;
    res.insertElementAt(item,res.size());

    String[] arr=getArray("alternateserver");
    for(int i=0;i<arr.length;i++)
    {
      String cmd=arr[i];
      int pos=cmd.indexOf(" ");
      if(pos>=0)
      {
        String host=cmd.substring(0,pos).trim();
        String port=cmd.substring(pos+1).trim();
        String pass="";
        pos=port.indexOf(" ");
        if(pos>=0)
        {
          pass=port.substring(pos+1).trim();
          port=port.substring(0,pos).trim();
        }
        item=new ServerItem();
        item.host=host;
        item.port=new Integer(port).intValue();
        item.pass=pass;
        res.insertElementAt(item,res.size());
      }
    }
    ServerItem[] ans=new ServerItem[res.size()];
    for(int i=0;i<ans.length;i++) ans[i]=(ServerItem)res.elementAt(i);
    return ans;
  }

  private StartupConfiguration getStartupConfiguration() throws Exception
  {
    String nick=getParameter("nick");
    if(nick==null) throw new Exception("Mandatory 'nick' parameter not provided");
    String name=getParameter("name");
    if(name==null) throw new Exception("Mandatory 'name' parameter not provided");
    String host=getParameter("host");
    if(host==null) throw new Exception("Mandatory 'host' parameter not provided");
    String pass=getParameter("password");
    if(pass==null) pass="";
    String sport=getParameter("port");
    if(sport==null) throw new Exception("Mandatory 'port' parameter not provided");
    int port=new Integer(sport).intValue();
    String altNick=getParameter("alternatenick");
    if(altNick==null) altNick=nick+"?";
    String alias=getParameter("serveralias");
    if(alias==null) alias="";

    ServerItem[] items=readServers(host,port,pass);
    String[] hosts=new String[items.length];
    int[] ports=new int[items.length];
    String[] passs=new String[items.length];
    for(int i=0;i<items.length;i++)
    {
      hosts[i]=items[i].host;
      ports[i]=items[i].port;
      passs[i]=items[i].pass;
    }
    return new StartupConfiguration(nick,altNick,name,passs,hosts,ports,alias,getArray("command"));
  }

}
